// Author: Torsten.Leitner@email.de
// (c) 2006/2007




#ifndef CONSTANTS_H_
#define CONSTANTS_H_

#include <complex>
#include <gsl/gsl_const_mksa.h>

#include "Definitions.h"

#include "logbuch.h"

#define PROGRAM_NAME "RHYNO"

// ***************************************************************************************************
/*/ *********************  LaserTypes  + VecPotTypes **************************************************
#define LASERTYPE_MONO  							0
#define LASERTYPE_GAUSS_NEAREST_POINT_METHOD    	1
#define LASERTYPE_GAUSS_LINEAR_INTERPOLATION    	2
#define LASERTYPE_GAUSS						    	2  // Standard for LASERTYPE_GAUSS is linear interpolation ...
// #define LASERTYPE_GAUSS_QUADRATIC_INTERPOLATION    3
#define VECPOT_NUMBER_OF_INTERPOLATION_POINTS       1e3
// */
//*
// *********************  VecPotTypes ****************************************************************
#define VECPOT_NEAREST_POINT_METHOD 	1
#define VECPOT_LINEAR_INTERPOLATION 	2
//#define VECPOT_QUADRATIC_INTERPOLATION 	3
//#define VECPOT_CUBIC_INTERPOLATION 		4
// ***************************************************************************************************
//*/
#define DEFAULTOPTIONFILE "defaults.opt" 

#define SYSTIME_OFFSET 1166022378	//Offset for time-measurement

#define StepsUntilDisplay 1000  //Number of Steps to integrate before intermediate result will be displayed

#define REGULARIZATION_CONST_TAUINTEGRAL 1e-32
#define EPS_ALIASING 1e-7
//#define ALIASING_WARNING "_ALIASED"
#define ALIASING_WARNING "-xxx"


//static const complex <double> IMAGUNIT(0,1) ;

static const double PI			= M_PI;

static const double SQRT2		= M_SQRT2;

static const double LIFETIMEtoFWHM	= 1.6651092223153953764125390080153010785579681396484;
					//    fwhm  = lifetime * ( 2*sqrt(ln(2)) ) --- value calculated with GNU-octave
static const double FWHMtoLIFETIME	= 0.6005612043932249122235589311458170413970947265625;
					// lifetime =   fwhm   / ( 2*sqrt(ln(2)) ) --- value calculated with GNU-octave


//
//   All hard-coded values for the physical constants are taken from:
// *******   http://physics.nist.gov/cuu/Constants/index.html   *******
// it's the US-NIST : "National Institute of Standards and Technology" 
//

static const double HPLANCK 					 	= 6.62606896e-34; // [Joule*sec]
static const double HPLANCK_EVOLT					= 4.13566733e-15; //  [eV s]
static const double HBAR 	 					 	= 1.054571628e-34; // [J s]
static const double HBAR_EVOLT					 	= 6.58211899e-16; // [eV s]

static const double CLIGHT	  						= 299792458; //[m/s]

static const double MUEZERO  						= 12.566370614e-7; //[Vs/Am]// 4pi x 10-7 = 12.566 370 614... x 10-7 [N/A^2];
static const double MUE0  	  						= MUEZERO;

static const double EPSILONZERO 					= 8.854187817e-12; // [C/Vm]=[F/m]
static const double EPSILON0  						= EPSILONZERO;

static const double ECHARGE	  						= 1.602176487e-19; // [C]

static const double EVOLT							= ECHARGE; // 1 [eV] = ( e/[C] ) [J] = 1.602176487e-19 [J]

static const double EMASS							= 9.10938215e-31; // [kg]

static const double KBOLTZMANN 						= 1.3806505e-23; // [Joule/Kelvin]

static const double HARTREE						 	= 4.35974394e-18;// [J] --- Hartree-Energy

static const double IP_HYDROGEN 					= 13.6; // [eV]
static const int     Z_HYDROGEN						= 1;
static const double IP_NEON     					= 21.6; // [eV]
static const int     Z_NEON						    = 10;
static const double LASER_EQUAL_ZERO				= 1e-100 ;

//static const double PREFACTOR_FUNDAMENTAL_SOURCE	= ECHARGE*ECHARGE/(EPSILONZERO*EMASS*CLIGHT*CLIGHT);
static const double PREFACTOR_FUNDAMENTAL_SOURCE	= 3.5411282e-14; //[m]
///    google:    (electron charge)^2 / ( (permittivity of free space)*(electron mass)*(speed of light)^2 )
/// 			      =  3.5411282 × 10-14 meters   --- mit TR gegengecheckt...

static const double TIR_ARGUMENT_CONSTANT = 20512714165.57189178466796875;  // [(eV)^-3] --- Prefactor for the argument...
										//  tir_prefac =  sqrt(32 * 511.99906e3) /(6.58211915e-16 * 2.99792458*e8) = 
										//			   = sqrt( 32 * (me/[eV/c2]) ) / ( hbar/[eVs] * c/[m/s] )
										//    X := { tir_prefac * (Ip/[eV])^3 }  /  abs[E(t)] ; 
										// tir(t)= Cnstar2 * (Ip/hbar) *  X^(2nstar-1) * exp(-x/3) ; 



static const double  MININTENSITY_TO_CALC_DIP =	1e10; // [W/cm2]
static const double  MINPOWER_TO_CALC_DIP    =	1e4*MININTENSITY_TO_CALC_DIP /(EPSILONZERO*CLIGHT);
// E^2[V2/m2] = I[W/m2] / epsilon0*c = 1e4*I[W/cm2] / epsilon0*c

// **************************************************************************************************
static const double   action_AUtoSI 	= HBAR;				    // [Js] 		[1au=hbar]
static const double   action_SItoAU 	= 1/action_AUtoSI;		// [au]
// **************************************************************************************************
static const double   energy_AUtoSI 	= HARTREE;				// [J]   		[1au=Eh]  
static const double   energy_SItoAU 	= 1/energy_AUtoSI;		// [au] 		Eh : Hartree-Energy

static const double   energy_AUtoeV 	= HARTREE/ECHARGE;		// [eV]		Eh[eV] = Eh[J=CV] / e[C]
static const double   energy_eVtoAU 	= 1/energy_AUtoeV;		// [au]
// **************************************************************************************************
static const double    VperM_AUtoSI 	= 5.14220632e11;		// [V/m]		[1au=Eh/(e*a0)]
static const double    VperM_SItoAU 	= 1/VperM_AUtoSI;		// [au]
// **************************************************************************************************
static const double     time_AUtoSI 	= 2.418884326505e-17;	// [s] 	  		[1au=hbar/Eh]
//static const double     time_AUtoSI 	= HBAR/HARTREE;	// [s] 	  		[1au(time) = hbar/Eh]
static const double     time_SItoAU 	= 1/time_AUtoSI ;	    // [au] 
// **************************************************************************************************
// **************************************************************************************************
static const double     omega_AUtoSI 	= time_SItoAU;	// [1/s]
static const double     omega_SItoAU 	= time_AUtoSI ;	    // [au] 
// **************************************************************************************************
static const double momentum_AUtoSI 	= 1.992851565e-24;   	// [kg m/s] 	[1au=hbar/a0]
static const double momentum_SItoAU 	= 1/momentum_AUtoSI;    // [au]
// **************************************************************************************************
static const double electric_dipole_moment_AUtoSI	= 8.47835281e-30; // [C m]  [1au=e*a_0]  
static const double electric_dipole_moment_SItoAU	= 1/electric_dipole_moment_AUtoSI;
// **************************************************************************************************
// **************************************************************************************************
static const double vecpot_AUtoSI	= HBAR/electric_dipole_moment_AUtoSI; // [s*V/m]  [1au = hbar/Eh * Eh/e*a0 = hbar/e*a0]  
static const double vecpot_SItoAU	= 1/vecpot_AUtoSI;
// **************************************************************************************************
// **************************************************************************************************
static const double meter_AUtoSI	= 0.52917720859e-10;// [m]
static const double meter_SItoAU	= 1/meter_AUtoSI;
// **************************************************************************************************
// **************************************************************************************************
//static const double hertz_AUtoSI	= 2.2523427369e23 ;// [Hz]  ---> stimmt nicht für omegas, keine Ahnung woher der wert kommt
//static const double hertz_SItoAU	= 1/hertz_AUtoSI;
// **************************************************************************************************
// **************************************************************************************************
static const double velocity_AUtoSI	= 2.1876912541e6;// [m/s]
static const double velocity_SItoAU	= 1/velocity_AUtoSI;
// **************************************************************************************************
// **************************************************************************************************
static const double Wpercm2_AUtoSI	= energy_AUtoSI*time_AUtoSI/(meter_AUtoSI*meter_AUtoSI)*1e-4; //[W/m2]*1e-4 = [W/cm2]
static const double Wpercm2_SItoAU	= 1/Wpercm2_AUtoSI;
// **************************************************************************************************
// **************************************************************************************************
//static const double meter_AUtoSI	= 0.52917720859e-10;// [m]
//static const double meter_SItoAU	= 1/meter_AUtoSI;
// **************************************************************************************************
static const double CLIGHT_AU		= velocity_SItoAU*CLIGHT; //[au]
// **************************************************************************************************
static const double MINPOWER_TO_CALC_DIP_AU = MINPOWER_TO_CALC_DIP * (VperM_SItoAU)*(VperM_SItoAU); // [V2/m2] -> [au]
// **************************************************************************************************

// from former definitions.h
// Definitions for the tau-integration
#define LOWER_TAU -10			// Lower Border of the tau-Integral
#define UPPER_TAU 0				// Upper ...
#define DTAU 1e-3				// Stepsize for Bode-Integration (and Naiv-Int.)
#define RELTAUINTERROR 1e-2		// Relative Error for GSL-Integration

#endif /*CONSTANTS_H_*/
