// Author: Torsten.Leitner@email.de
// (c) 2006/2007


#include "gnuplot.h"


// *************************************************************************
inline double gplotfunc(const double r){
      return sin(r*r+1.9)-10*log(r+0.3)+r*r-r*(r-2);
}
// *************************************************************************
// ***      gnuplot funktion fuer c++ ... inkl. png-plot moeglichkeit
// *************************************************************************
int vecgplotplotxy(vector<double> xx, vector<double> yy, 
            double delay, int logscale, int terminaltype ,
            char *xlabel , char *ylabel, char *title, char *style )
{
// Plotting with selection of terminaltype ...    

// Even numbers indicate X11 and file output (of filetype nr-1)
// Odd  numbers indicate file output only
// terminaltype =
// 0 -> x11
// 1 -> png
// 2 -> png + x11
// and
// logscale =
// 0 -> no logscale
// 1 -> log x
// 2 -> log y
// 3 -> log x, log y


    int N;
    N=xx.size(); if ( yy.size() < xx.size() ) {N = yy.size();}
    double          x[N] ;
    double          y[N] ;
    int             i ;
// /*
    for (i=0 ; i<N ; i++) {
        x[i] = xx.at(i);
        y[i] = yy.at(i);
    }
// */
    gnuplot_ctrl * h ;
    h = gnuplot_init() ;

    
    gnuplot_setstyle(h, style) ;
    gnuplot_set_xlabel(h, xlabel) ;
    gnuplot_set_ylabel(h, ylabel) ;
    
    if (logscale == 1) { gnuplot_cmd(h, "set logscale x");
                       }
    if (logscale == 2) { gnuplot_cmd(h, "set logscale y");
                       }
    if (logscale == 3) { gnuplot_cmd(h, "set logscale x");
                         gnuplot_cmd(h, "set logscale y");
                       }

// Plotting with selection of terminaltype ...    

// Even numbers indicate X11 and file output (of filetype nr-1)
// Odd  numbers indicate file output only
// terminaltype =
// 0 -> x11
// 1 -> png
// 2 -> png + x11
    
       // normal X11-Plot
    if (terminaltype == 0) {  cout << "Terminal: X11 ... waiting ..." << endl;
    						  gnuplot_cmd(h, "set terminal X11"); 
                              gnuplot_plot_xy(h, x, y, N, title) ;  
                              sleep(int(delay)) ;
                          }

       // only  png-Plot (title.png)
    if (terminaltype == 1) {     cout << "Terminal: png ..." << endl;
    	 						 gnuplot_cmd(h, "set terminal png");
    							 gnuplot_cmd(h, "set output \"term1.png\"");
                                 gnuplot_plot_xy(h, x, y, N, title) ;
                                  
                           }
                              
                              
       // X11-Plot + png-Plot (title.png)
    if (terminaltype == 2) {     cout << "Terminal: png + X11 ... waiting ..." << endl;
    							 gnuplot_cmd(h, "set terminal X11"); 
                                 gnuplot_plot_xy(h, x, y, N, title) ;  
                                  sleep(int(delay)) ;               
                                 gnuplot_resetplot  	(   	h  	);
                                 
        
                                 gnuplot_cmd(h, "set terminal png");
                                 gnuplot_cmd(h, "set output \"term2.png\"");
                                 gnuplot_plot_xy(h, x, y, N, title) ;
                                                               
                          }

// */    
    
    gnuplot_close(h) ;
    return 0;
}
// *************************************************************************	
int gplotplotxy(int N, double x[], double y[], 
            double delay, int logscale, int terminaltype ,
            char *xlabel , char *ylabel, char *title, char *style ,char *filename, int filenr)
{
// Plotting with selection of terminaltype ...    

// Even numbers indicate X11 and file output (of filetype nr-1)
// Odd  numbers indicate file output only
// terminaltype =
// 0 -> x11
// 1 -> png
// 2 -> png + x11
// and
// logscale =
// 0 -> no logscale
// 1 -> log x
// 2 -> log y
// 3 -> log x, log y

char string[80];	// helpvariable 

    gnuplot_ctrl * h ;
    h = gnuplot_init() ;

    gnuplot_setstyle(h, style) ;
    gnuplot_set_xlabel(h, xlabel) ;
    gnuplot_set_ylabel(h, ylabel) ;
    
    gnuplot_cmd(h, "set yrange [0:1]");
    
    
    if (logscale == 1) { gnuplot_cmd(h, "set logscale x");
                       }
    if (logscale == 2) { gnuplot_cmd(h, "set logscale y");
                       }
    if (logscale == 3) { gnuplot_cmd(h, "set logscale x");
                         gnuplot_cmd(h, "set logscale y");
                       }
                       
       // normal X11-Plot
    if (terminaltype == 0) {  cout << "Terminal: X11 ... waiting ..." << endl;
    						  gnuplot_cmd(h, "set terminal X11"); 
                              gnuplot_plot_xy(h, x, y, N, title) ;  
                              sleep(int(delay)) ;
                          }

sprintf (string, "set output \"%s%i.png\"",filename, filenr);
       // only  png-Plot (title.png)
    if (terminaltype == 1) {     cout << "Terminal: png ..." << endl;
    	 						 gnuplot_cmd(h, "set terminal png");
    	 						 gnuplot_cmd( h,  string  );
                                 gnuplot_plot_xy(h, x, y, N, title) ;
                           }
       // X11-Plot + png-Plot (title.png)
    if (terminaltype == 2) {     cout << "Terminal: png + X11 ... waiting ..." << endl;
    							 gnuplot_cmd(h, "set terminal X11"); 
                                 gnuplot_plot_xy(h, x, y, N, title) ;  
                                  sleep(int(delay)) ;               
                                 gnuplot_resetplot  	(   	h  	);
                                 gnuplot_cmd(h, "set terminal png");
                                 gnuplot_cmd(h, string );
                                 gnuplot_plot_xy(h, x, y, N, title) ;
                          }

// */    
    
    gnuplot_close(h) ;

    return 0;
}
// *************************************************************************	
int test_gnuplot(void)
{

vector<double> x,y;

int N=1000;
double dr=1e-1;

double r=0;
for(int i=0;i<N;i++){
      x.push_back(r);
      y.push_back(gplotfunc(r));
      r+=dr;
}
int logscale=0;
int terminaltype=0;
char *xlabel="x";
char *ylabel="y";
char *title ="f(x)";
char *style="points";
double delay=5;
terminaltype=0;
//gplotplotxy(x,y,delay,logscale ,terminaltype, xlabel ,ylabel, title ,style );
//gplotplotxy(x,y,delay,logscale ,1, xlabel ,ylabel, title ,style );
vecgplotplotxy(y,x,delay,logscale ,2, xlabel ,ylabel, title ,style );

return 0;
}


gnuplot::gnuplot()
{
}

gnuplot::~gnuplot()
{
}
