// Author: Torsten.Leitner@email.de
// (c) 2006/2007


#include "twodee.h"



// Simulation of the Field in Cylindrical-Coordinates...

// First: Radial Symmetrie is assumed ->  (d^2)phi / d(phi^2) = 0

// *********************************************************************************** //
twodee::twodee()
{
}
// *********************************************************************************** //
twodee::twodee(double zdz, int zNr, double zdr)
{
	SetParameter(zdz,zNr,zdr);
}

twodee::~twodee()
{
}

void twodee::SetParameter(double zdz, int zNr, double zdr)
{
	Nr=zNr;
////// 		prefactor of the (d/dz)-Term :  c/2*i*omega = -0.5*i*c/omega

//	prefactor = Complex(0,-1); 
	dz=zdz;
	dr=zdr; 
	
///*	
//	u = new Complex [Nr];  
//	for (int i=0;i<Nr;i++){ u[i] = field.at(i); }
//	u[Nr]=0; //Border = 0
	
//	uvec = field;
//*/	
//      r = new double[Nr];   for (int i=0;i<Nr;i++) { r[i] = i*dr;   }
	
	
	alpha = new Complex[Nr];
	beta  = new Complex[Nr];
	gamma = new Complex[Nr];
	b     = new Complex[Nr];
	
}
/******************************************************************************/

/******************************************************************************/
int twodee::info()
{  // searches maximum of a peak and determines the FWHM ...
	int j;
	
	double y[Nr];
	
 	double max=0; int imax=0;
 	double e,d=1e8;
 	double fwhm;
 	int j_hwhm = 0;

 // Search Maximum ...
	for (j=0; j<Nr; j++)
		{  	
			y[j]=abs(u[j])*abs(u[j]);
			if (y[j]>max) { max=y[j]; imax=j; }
		}
// Determine FWHM ...
	for (j=0; j<Nr; j++)
		{  	
			e = abs( 2*y[j] - max ); // e ist Unterschied zw. y[i] und ymax/2
			if ( e < d ) { d = e; j_hwhm = j; } // Hier wird (2*y[i]-max) minimiert und Pos. des Minimums notiert.
		}
		fwhm = 2 * r[j_hwhm];
		
	cout << "Maximum:  y = " << max << "  at r = " << r[imax] << "   FWHM = " << fwhm << endl;
		
	return 0;
}	
/******************************************************************************/

/******************************************************************************/
int twodee::show_plot(char *title, char *filename, int filenr)
{
	return 	show_plot("r", "|E(r)|^2" , title, "points" ,filename, filenr, 0,1,0);
}
/******************************************************************************/
int twodee::show_plot(char *xlabel , char *ylabel, char *title, char *style ,char *filename, int filenr, 
					  int logscale, int terminaltype, double delay)
{	
	double y[Nr];
	 r = new double[Nr];   for (int i=0;i<Nr;i++) { r[i] = i*dr;   }
	for (int j=0; j<Nr; j++)
		{  	
			y[j]=abs(u[j])*abs(u[j]);
		}
//gplotplotxy(N,x,y,delay,logscale ,terminaltype, xlabel ,ylabel, title ,style ,filename, 10);
    gplotplotxy( Nr , r , y,  delay,logscale ,terminaltype,   xlabel , ylabel, title , style ,filename, filenr);
	
	// Nx=nn;
		
	return 0;
}
/******************************************************************************/
int twodee::evolve(Complex *field, Complex *source, double INomega, int steps)
{
	int error = -1;
	
	u=field;
//	src=source;
	omega = INomega;
	
	for (int i=0;i<steps;i++) {
		error = evolve_onestep();
	}

	return error;	
}

/******************************************************************************/
/*
/ ****************************************************************************** /
int twodee::evolve(vector<Complex > *field, Complex *source, double INomega, int steps)
{
	int error = -1;
	
	u=field;
//	src=source;
	omega = INomega;
	
	for (int i=0;i<steps;i++) {
		error = evolve_onestep();
	}

	return error;	
}

/ ******************************************************************************/
int twodee::evolve_onestep()

// Wahl von dz ist micht ganz klar... fuer Nr=1000 ist dz = dr/(100*Nr); gut Wahl!
//          Generelle Regel: ???

{
	// u has one Element more: u[N]=0, so we have Dirichlet-Boundary-Conditions with running over all inner Points (j<N)
	// u[N]=0;
//  			              (beta0 , gamma0, 0			   				    )
//	Matrixequation:  A*u =    (alpha1, beta1 , gamma1 , 0			     	    )*u  =   b(old_u) ;
//        					  ( 0	, alpha2, beta2  , gamma2 , 0				)
//					          ( ............................................... )
//
// This is a Crank-Nicolson-Routine (CrN):
// un: new u --- uo: old u ,,,, a=alpha,b=beta,g=gamma
//
// ->  un(j) + { a(j)un(j-1) + b(j)un(j) + g(j)un(j+1) } =  uo(j) - { a(j)uo(j-1) + b(j)uo(j) + g(j)uo(j+1) }
//
//	In textbooks they usually define it different... here: all prefactors, like (1/2) and the discretization 
//														   are include in the a,b,g s ...
//	I also switched the signs ... Usually there's a Minus on the left- and a plus on the right-hand-side
//			->  D = (1/2)*( dz / h^2 ) ; .... in this prog: h=dr; 
//  Note that dz is refered as dt in the diffusion-equations in textbooks ...
// 




	double aj; 			// Short-term-memory ...
	Complex cc; // Complex-Short-term-memory ...
	Complex ComplexOne(1,0);
	int j;
	
	//double dD = 0.5 * dz / (dr*dr);  // dD = tstep/(2*h^2)  .... 1/2 wg. CrankNic
	//prefactor = Complex(0,-0.5*CLIGHT/omega);
	//Complex D = dD * prefactor; 
	
	Complex D(0,-0.5*CLIGHT/omega);  	
	D*= 0.5 * dz / (dr*dr);

	u[Nr-1]=0;
	
	Complex ljm,lj,ljp;

	double rp=0,rm=0;  // rp = r(j+0,5) ... rm = r(j-0,5)

// This loop applies the new RightHandSide (rhs) b  and the non-zero matrixelements: alpha[j],beta[j],gamma[j]		
 
for (j=0;j<Nr-1;j++) 
		{
		      rm = rp;
		      rp = (j+0.5)*dr;  // equal spaced
		       
//		      rp = (r[j] + r[j+1])/2; // for a given vector r[j]
		      
			aj = 2 / ( rp*rp - rm*rm );
	
	// for a given vector of r[j]		
//			ljm      = aj * ( rm / ( r[j] - r[j-1] ) );     //ljm = Laplace(j,j-1)
//			ljp      = aj * ( rp / ( r[j+1] - r[j] ) );     //ljp = Laplace(j,j+1) 		

	// for equally spaced r[j]					
			ljm      = aj * ( rm / dr);					    //ljm = Laplace(j,j-1)
			ljp      = aj * ( rp / dr);					    //ljp = Laplace(j,j+1) 
	// alwqys the same					
			lj       = - ( ljm + ljp );  // +  f(j,angle)   //lj  = Laplace(j , j)
				
			alpha[j]  = -D * ljm;
			gamma[j]  = -D * ljp;
			beta [j]  =  ComplexOne - D* lj;
			
			b[j] 	 = u[j] + D * ( ljm*u[j-1] + lj*u[j] + ljp*u[j+1] ) ;
		} 


// Forward Recursion to get rid off all Elements lower than the diagonal...
//cout <<" Start Forward ..."<<endl;
	for (j=1;j<Nr-1;j++) 
		{ 
			cc = alpha[j] / beta[j-1];
			
			beta[j] = beta[j] -  cc * gamma[j-1];
			b[j] 	  = b[j] 	  -  cc * b[j-1];		  
		}
		
// Backward Recursion ...
//	Still: u[Nr]=0; --> Outer Rim
//	uneu[Nr-1] = 0;
//	uneu[Nr-2] = b[Nr-2] / beta[Nr-2];
	u[Nr-1] = 0;
	u[Nr-2] = b[Nr-2] / beta[Nr-2];		
	for (j=Nr-3;j>=0;j--) 
		{ 
//			uneu[j] = ( b[j] - gamma[j]*uneu[j+1] ) / beta[j];
			u[j] = ( b[j] - gamma[j]*u[j+1] ) / beta[j];
		}
// finished ... evolve_onestep();	

//	for (j=0;j<Nr;j++) { u[j] = uneu[j]; }
	

//for (j=0;j<Nr;j++) { cout << j << uneu[j] << endl; } cout << "\n ******************** \n " <<endl;

	return 0;
}
/******************************************************************************/

