// Author: Torsten.Leitner@email.de
// (c) 2006/2007



#include "options.h"

// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
int SetStandardInputParameters(InputParameter *in)
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
{ // Standards partly from the experiment described in the priori paper
// Flags
	in->istimestamp = false;
// Files	
	in->outfile = "Unnamed";
	in->optfile = "";
// The fundamental Laser	
	in->lam			= 740e-9; // [m] ... Ti:Sa - Laser
	in->omega0		= 2*PI*CLIGHT / in->lam; // [1/s]	
	in->Intensity   = 8e14; // Standard: 8e14 [W/cm^2]	
	in->E0			= 1e2 * sqrt( in->Intensity / (EPSILONZERO*CLIGHT) ); // [V/m] 
					// E0 = sqrt(I[W/m2]/eps0*c)=1e2*sqrt(I[W/cm2]/eps0*c) 
	in->Up			= ECHARGE*ECHARGE*in->E0*in->E0 / ( 4*in->omega0*in->omega0*EMASS );  //[J]
	in->fwhm	= 30*1e-15; // 30fs
	in->lifetime	= FWHMtoLIFETIME * in->fwhm; // -> 18.0168 fs
	in->waist0   	= 25*1e-4 ;  // [m] - Size of the focal spot
	in->zfocus	    = 0;        // [m] - Position of the focal spot,
								// relative to the beginning of the interaction region

// The gas
	in->ZAtom  =  Z_NEON;  // -> Neon
	in->Ip     = IP_NEON;  // [eV] -> Neon -- typically (5-20)[lew] eg. 14.4 [lew]
	in->Pressure	= 53.328  ; // [mbar]  
	in->Temperature	= 297.12  ; // [Kelvin]
	
// scale from [mbar] to  1[Pa]   = 1[N/m2]  --- [1/m3] = 1e-6[1/cm3] --- [kb] = [J/K]
//					  -> 1[mbar] = 1e2[Pa] = 1e2[N/m2]
//  n0 = p/kT --> [n0] = [mbar]/[(J/K)*K] = 1e2[N/m2]/[Nm] = 1e2[1/m3] = 1e-4[1/cm3] ;
	in->AtomDensity	= 1e2 * in->Pressure / ( KBOLTZMANN * in->Temperature ); // [1/m3]	;

	
	in->qcut        = (3.17*in->Up+ECHARGE*in->Ip) / (HBAR*in->omega0) ;
	

	in->nPics = 2; // No. of created mesh.dat files

// Simulationparams I
					// Propagation Parameters
	in->nsteps = 10;		// Steps between Source-Evaluations
	in->nruns  = 1;			// Number  of    Source-Evaluations
	in->dz	   = 1e2*(in->lam);	// Propagation stepsize  ( dz = c*dt ) 
	
	in->Nr	   = 101; 	// Spatial Grid
	in->rmax   = 5 * (in->waist0);
	in->dr	   = in->rmax / (in->Nr -1);
		
	
	in->Ntw= 8192;	// Fundamental Time/Omega Grid
//	in->dt
//	in->tmin
//	in->tmax
//	in->dw
//	in->wmin
//	in->wmax;	
		
		
	in->Ntw1= in->Ntw;	// Fundamental Time/Omega Grid
//	in->dt1
//	in->t1min
//	in->t1max
//	in->dw1
//	in->w1min
//	in->w1max;
	
	in->Ntwh= in->Ntw;	// Harmonic Time/Omega Grid
//	double dth,thmin,thmax;
//	double dwh,whmin,whmax;


// Simulation Params. II
	in->Q    = 300 ;        // Max.Sampled frequency in multiples of omega0
	in->L 	 = 3;		// Min. Size of time window in measures of laser lifetime tau
	in->ppoc = 2 * in->Q;   // Points per optical cycle
	
	in->Noc	 = 3;		// Number of optical cycles to be integrated	
	in->Npoints2integrateXt = int( in->Noc * in->ppoc + 0.5 ); // Number of points to integrate nl dip.moment
	in->threshold  =  MININTENSITY_TO_CALC_DIP; // 1e10 [W/cm2]


// Old params n stuff	
	in->Tmin = 0;
	
	in->abs_omega_error = 1e-2;
	in->taumin = 0; // Min.IntegrationTime in opt.Cycles of the fundamental (2pi each)
	in->taumax = 1 * 2 * PI; // Max.IntegrationTime in opt.Cycles of the fundamental (2pi each)
	in->dtau = 2 * PI * 1e-3;
	
	in->vecpot_number_of_interpolation_points = int(1e5);
	
	return 0;
}  
 // ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
  
  
 // ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 // ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
int handle_all_options(int argc, char **argv, InputParameter *inpar)
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
{
       /* 1. CREATE AN OBJECT */
        AnyOption *opt = new AnyOption();

        /* 2. SET PREFERENCES  */
        //opt->noPOSIX(); /* do not check for POSIX style character options */
        opt->setVerbose(); /* print warnings about unknown options */
        opt->autoUsagePrint(true); /* print usage for bad options */


		/* 3. SET THE USAGE/HELP   */ 
		/* 4. SET THE OPTION STRINGS/CHARACTERS */
        
        opt->addUsage( "" );
        opt->addUsage( "Usage: " );
//      opt->addUsage( "" );

	/* by default all  options  will be checked on the command line and from option/resource file */
    /* for options that will be checked only on the command and line not in option/resource file */
        opt->setCommandFlag(  "help", 'h' ); /* a flag (takes no argument), supporting long and short form */   
        opt->addUsage ( "     --help                  Prints this help " );

        opt->addUsage ( "" );
    	opt->addUsage ( " The following options are commanddline and file options;" );
    	opt->addUsage ( "  -In an option file (e.g. defaults.opt) write: \"option : value\")" );
    	opt->addUsage ( "  -Any line starting with \"#\" is a commentline " );
        opt->addUsage ( "  -Do not add comments to the end of a line with an option" );
        opt->addUsage ( "" );

        opt->setOption( "nPics"); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( "     --nPics                 Set Number of output-mesh.dat-files");
        opt->addUsage ( "                              for further examination and plotting");
        
        opt->setOption( "nsteps"); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( "     --nsteps                Set Number of steps till Source Evaluation");
        
        opt->setOption( "nruns"); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( "     --nruns                 Set total Number of Source Evaluations");
                        
        opt->setOption( "lam", 'l' ); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( " -l  --lam   lambda[m]       Set wavelength in [m]");
        
        opt->setOption( "Intensity", 'I' ); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( " -I  --Intensity  [Int]      Set Intensity in [W/cm2]");
 
	opt->setOption( "lifetime"); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( "     --lifetime [tau]        Lifetime tau of the Laser [sec] ");
        opt->addUsage ( "                               ->  E(t) ~ exp[-(t/tau)^2]    ");
        opt->addUsage ( "                              option fwhm overrides lifetime ");

	opt->setOption( "FWHM",'d'); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( " -d  --FWHM [duration]       Duration of the laser pulse as FWHM in [sec]");
        opt->addUsage ( "                               fwhm = 2*sqrt(ln(2)) * lifetime ");

	opt->setOption( "waist0"); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( "     --waist0 [W0]           Waist at focus in [m]      ");
        
        opt->setOption( "zfocus"); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( "     --zfocus [zf]           Position of the focal spot relative");
        opt->addUsage ( "                              to the beginning of the interaction");
        opt->addUsage ( "                              region (e.g. capillary)  in [m]  ");
        
        opt->setOption( "Nr"); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( "     --Nr  [nr]              Number of radial (spatial) grid points");
        
        opt->setOption( "rmax"); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( "     --rmax [m]              Radius of the spatial grid");
        
        opt->setOption( "dz"); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( "     --dz [m]                Propagation stepsize  ( dz = c*dt ) ");
        
        opt->setOption( "ZAtom" ); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( "     --ZAtom Z               Set atom species via the atomic number");
           	    
  	opt->setOption( "Ip" ); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( "     --Ip  [Ip]              Set wavelength in [eV]");
        
        opt->setOption( "Pressure", 'p' ); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( " -p  --Pressure  [p]         Set Pressure in [mbar]");       
        
 	opt->setOption( "Temperature", 'T' ); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( " -T  --Temperature  [T]      Set Temperature in [Kelvin]");
        
        opt->setOption( "AtomDensity"); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( "     --AtomDensity  [n0]     Set atom density in [1/m3]=1e-6[1/cm3]");     
       
        opt->setOption( "Qmax", 'Q' ); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( " -Q  --Qmax                  Max.sampled frequency in units [w0]");

        opt->setOption( "Lmin", 'L' ); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( " -L  --Lmin                  Min.size of the time window in units");
        opt->addUsage ( "                              of the laser's lifetime [tau] ");
        
        opt->setOption( "Ntw", 'N' ); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( " -N  --Ntw                   Number of Points in time           ");

	opt->setOption( "Noc" ); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( "     --Noc                   Number of opt. cycles , over which the");
        opt->addUsage ( "                              dipole moment is integrated (double)");

	opt->setOption( "threshold",'t' ); /* an option (takes an argument), supporting long and short form */
	opt->setOption( "Intmin" ); 
	opt->addUsage ( " -t  --threshold             Min. peak intensity (threshold)");
        opt->addUsage ( "     --Intmin                 to start the calculation of a");
        opt->addUsage ( "                              single atom spectrum   [W/cm2]");

	opt->setOption( "optfile" ); /* an option (takes an argument), supporting long form only */
        opt->addUsage ( "     --optfile optfile       Alternative option file" );
        
        opt->setOption( "outfile", 'o' ); /* an option (takes an argument), supporting long and short form */
        opt->addUsage ( " -o  --outfile outputname    Mainname of output files" );
        opt->addUsage ( "                              (suffixes added automatically)" );
 		       
        opt->setFlag  ( "timestamp" );   /* a flag (takes no argument), supporting long form only */ 
	opt->addUsage ( "     --timestamp             Adds a timestamp to the output " );
        opt->addUsage ( "" );
        opt->addUsage ( "" );
      
// *********************************************************************************************************************
// *********************************************************************************************************************  

// *********************************************************************************************************************  
//  begin: OLD OPTIONS ...
// *********************************************************************************************************************  
/*
        opt->addUsage ( "  ");opt->addUsage ( " OLD OPTIONS --- TO BE REMOVED ");opt->addUsage ( "  ");  

        opt->setOption( "Tmin" ); // an option (takes an argument), supporting long and short form 
        opt->addUsage ( "     --Tmin                  Starttime in [sec]");
        
        opt->setOption( "Tmax" ); // an option (takes an argument), supporting long and short form 
        opt->addUsage ( "     --Tmax                  Stoptime in  [sec]");
        
       	opt->setOption( "domega", 'd' ); // an option (takes an argument), supporting long and short form 
        opt->addUsage ( " -d  --domega                Abs.error of output the spectrum [w0]");
        
        opt->setOption( "taumin" ); // an option (takes an argument), supporting long and short form 
        opt->addUsage ( "     --taumin                Lower border of tau-integration");
        
        opt->setOption( "taumax" ); // an option (takes an argument), supporting long and short form 
        opt->addUsage ( "     --taumax                Upper border, both  in opt.cycles");
		
		opt->setOption( "dtau" ); // an option (takes an argument), supporting long and short form 
        opt->addUsage ( "     --dtau                  tau-Stepwidth in opt.cycles");
        
        opt->setOption( "LaserGauss" ); // an option (takes an argument), supporting long and short form 
        opt->addUsage ( "     --LaserGauss            Set Lasertype Gaussian ...");
                               
        opt->addUsage ( "" );
//*/
// *********************************************************************************************************************  
//   end: OLD OPTIONS.
// *********************************************************************************************************************  
	
/* for options that will be checked only from the option/resource file */
    	opt->addUsage ( " Options only valid in option file: (write \"option : value\")" );
        opt->addUsage ( "" );
        opt->setFileOption(  "title" ); /* an option (takes an argument), supporting only long form */
        opt->addUsage ( " title : Name                Sets the title of the Simulation" );
        
        // Newline at end of Usage
        opt->addUsage( "" );
        
        
        /* 5. PROCESS THE COMMANDLINE AND RESOURCE FILE */

	/* read options from a  option/resource file with ':' separated opttions or flags, one per line */
        opt->processFile( DEFAULTOPTIONFILE );  
	if ( opt->hasUnknownFileOptions()) { cout << "Error in default option file: \"defaults.opt\"" << endl; return 3; }
	/* if alternative option file ->  process it */
	  if( opt->getValue( "optfile" ) != NULL ) {
		inpar->optfile = opt->getValue( "optfile" );
       	 	cout << setw(28)<< "Alternative OptFile =  " << inpar->optfile << " \n " << endl;
		opt->processFile( inpar->optfile.c_str() );
		}
	/* go through the command line and get the options  */
        opt->processCommandArgs( argc, argv );
        
        if( opt->getFlag( "help" ) || opt->getFlag( 'h' ) ) {   //   flag
                opt->printUsage();
                return 1;
        }
    	if ( opt->hasUnknownCmdOptions() ) { return 2; }  //     

	if ( opt->hasUnknownFileOptions()) { return 3; }
/*	if( ! opt->hasOptions()) { // print usage if no options 
                opt->printUsage();
//	        delete opt;
//		return;
	}
// hasOptions = TRUE, only if commandline args are given ...
// */

        /* 6. GET THE VALUES */
        
        
        inpar->title = string(argv[0]);  //Standard title = Pogramname 	
		if( opt->getValue( "title" ) != NULL ) {inpar->title = opt->getValue( "title" );}
		cout << setw(28)<< "title =  "<< "  " << inpar->title << endl ;
        
        if( opt->getFlag( "timestamp") ) {   //   flag
                inpar->istimestamp = true;
                cout << setw(28)<<" Timestamp =  " << setw(7) << " TRUE "<<endl;
        }
                
       	if( opt->getValue( "outfile" ) != NULL ) {
       		inpar->outfile = opt->getValue( "outfile" );
       	 	cout << setw(28)<< " Main output name =  "<< "  " << inpar->outfile << endl ;
       	}
       	
       	if( opt->getValue( "nPics" ) != NULL ) {
       		inpar->nPics = atoi(opt->getValue( "nPics" ));
       	 	cout << setw(28) << " nPics set to:  " << setw(7) << inpar->nPics << endl ;
       	}
       	
       	if( opt->getValue( "nsteps" ) != NULL ) {
       		inpar->nsteps = atoi(opt->getValue( "nsteps" ));
       	 	cout << setw(28) << " nsteps set to:  " << setw(7) << inpar->nsteps << endl ;
       	}
       	
       	if( opt->getValue( "nruns" ) != NULL ) {
       		inpar->nruns = atoi(opt->getValue( "nruns" ));
       	 	cout << setw(28) << " nruns set to:  " << setw(7) << inpar->nruns << endl ;
       	}
       	
       	 	if( opt->getValue( "ZAtom" ) != NULL ) {
       		inpar->ZAtom = atoi(opt->getValue( "ZAtom" ));
       	 	cout << setw(28) << " ZAtom set to:  " << setw(7) << inpar->ZAtom << endl ;
       	}
       	
       	if( opt->getValue( "Ip" ) != NULL ) {
       		inpar->Ip = atof(opt->getValue( "Ip" ));
       	 	cout << setw(28) << " Ip set to:  " << setw(7) << inpar->Ip << " [eV]" << endl ;
       	}
       	
       	if( opt->getValue( "Pressure" ) != NULL ) {
       		inpar->Pressure = atof(opt->getValue( "Pressure" ));
       	 	cout << setw(28) << " Pressure set to:  " << setw(7)<< inpar->Pressure << " [mbar]" << endl ;
       	}
       	if( opt->getValue( "Temperature" ) != NULL ) {
       		inpar->Temperature = atof(opt->getValue( "Temperature" ));
       	 	cout << setw(28) << " Temperature set to:  " << setw(7)<< inpar->Temperature << " [Kelvin]" << endl ;
       	}
   
       	if( opt->getValue( "AtomDensity" ) != NULL ) {
       		inpar->AtomDensity = atof(opt->getValue( "AtomDensity" ));}
       	else { inpar->AtomDensity	= 1e2 * inpar->Pressure / ( KBOLTZMANN * inpar->Temperature );} 	// [1/m3]	;
       	cout << setw(28) << " Atom density n0 set to:  " << setw(7)<< inpar->AtomDensity << " [1/m3]=1e-6[1/m3]" << endl ;
       	
       	
       	if( opt->getValue( "lam" ) != NULL ) {
       		inpar->lam = atof(opt->getValue( "lam" ));
       	 	cout << setw(28) << " Lambda set to:  " << setw(7) << inpar->lam << " [m]" << endl ;
       	 	inpar->omega0    = 2*PI*CLIGHT/inpar->lam;
       	 	inpar->dz	         = 1e2*(inpar->lam);	// Propagation stepsize  ( dz = c*dt ) 
       	}
       	
       	if( opt->getValue( "Intensity" ) != NULL ) {
       		inpar->Intensity = atof(opt->getValue( "Intensity" ));
       	 	cout << setw(28) << " Intensity set to:  " << setw(7) << inpar->Intensity << " [W/cm2]" << endl ;
       	 	inpar->E0 = 1e2 * sqrt( inpar->Intensity / (EPSILONZERO*CLIGHT) ); // [V/m] 
				// E0 = sqrt(I[W/m2]/eps0*c)=1e2*sqrt(I[W/cm2]/eps0*c) 
		}
		// Always re-Set Up at this point, after new Intensity and new Lambda is known ... 
		// and cut off after I,lam, Ip
		inpar->Up			= ECHARGE*ECHARGE*inpar->E0*inpar->E0 / ( 4*inpar->omega0*inpar->omega0*EMASS );
		inpar->qcut	        = (3.17*inpar->Up+ECHARGE*inpar->Ip) / (HBAR*inpar->omega0) ;
       	
        if( opt->getValue( "lifetime" ) != NULL ) {
       		inpar->lifetime = atof(opt->getValue( "lifetime" ));
       		inpar->fwhm     = LIFETIMEtoFWHM * inpar->lifetime;
       	 	cout << setw(28) << " Lifetime set to:  " << setw(7) << inpar->lifetime << " [sec]" << endl ;
       	 	cout << setw(28) << " -> FWHM  set to:  " << setw(7) << inpar->fwhm << " [sec]" << endl ;
       	}

	if( opt->getValue( "FWHM" ) != NULL ) {
       		inpar->fwhm     = atof(opt->getValue( "FWHM" ));
		inpar->lifetime = FWHMtoLIFETIME * inpar->fwhm;
       	 	cout << setw(28) << "FWHM  set to:  " << setw(7) << inpar->fwhm << " [sec]" << endl ;
       	}

       	if( opt->getValue( "waist0" ) != NULL ) {
       		inpar->waist0 = atof(opt->getValue( "waist0" ));
       	 	cout << setw(28) << " waist0 set to:  " << setw(7) << inpar->waist0 << " [m]" << endl ;
       	}
       	
       	if( opt->getValue( "zfocus" ) != NULL ) {
       		inpar->zfocus = atof(opt->getValue( "zfocus" ));
       	 	cout << setw(28) << " zfocus set to:  " << setw(7) << inpar->zfocus << " [m]" << endl ;
       	}
       	
       	if( opt->getValue( "Nr" ) != NULL ) {
       		inpar->Nr = atoi(opt->getValue( "Nr" ));
       	 	cout << setw(28) << " Nr set to:  " << setw(7) << inpar->Nr << endl ;
       	}
       	if( opt->getValue( "rmax" ) != NULL ) {
       		inpar->rmax = atof(opt->getValue( "rmax" ));
       	 	cout << setw(28) << " rmax set to:  " << setw(7) << inpar->rmax << " [m]" << endl ;
       	}
       	inpar->dr = inpar->rmax / (inpar->Nr -1);  // recalculate
       	
       	if( opt->getValue( "dz" ) != NULL ) {
       		inpar->dz = atof(opt->getValue( "dz" ));
       	 	cout << setw(28) << " dz set to:  " << setw(7) << inpar->dz << " [m]" << endl ;
       	}
       	
       	if( opt->getValue( "Tmin" ) != NULL ) {
       		inpar->Tmin = atof(opt->getValue( "Tmin" ));
       	 	cout << setw(28) << " Tmin set to:  " << setw(7) << inpar->Tmin << " [opt.cycles]" << endl ;
       	 	inpar->Tmin = 2 * PI * inpar->Tmin;
       	}
       	
       	if( opt->getValue( "Qmax" ) != NULL ) {
       		inpar->Q = atof(opt->getValue( "Qmax" ));
		inpar->ppoc = 2.0 * inpar->Q;
       	 	cout << setw(28) << " Q set to:  " << setw(7) << inpar->Q << " [omega0]  => " << inpar->ppoc << " ppoc" << endl ;
       	}

       	if( opt->getValue( "Lmin" ) != NULL ) {
       		inpar->L = atof(opt->getValue( "Lmin" ));
       	 	cout << setw(28) << " L set to:  " << setw(7) << inpar->L << " [tau]" << endl ;
       	}

	
       	if( opt->getValue( "Noc" ) != NULL ) {
       		inpar->Noc = atof( opt->getValue( "Noc") );
		inpar->Npoints2integrateXt = int( inpar->Noc * inpar->ppoc +1 );
       	 	cout << setw(28) << " Noc set to:  " << setw(7) << inpar->Noc << " [opt.cycles]" << endl ;
		cout << setw(28) << " => points2int Xt:  " << setw(7)<< inpar->Npoints2integrateXt << endl ;
       	}


	if( opt->getValue( "Intmin") ) {
       		inpar->threshold = atof( opt->getValue( "Intmin") );
       	 	cout << setw(28) << " Threshold set to:  " << setw(7) << inpar->threshold << " [W/cm2]" << endl ;
       	}
	if( opt->getValue( "threshold") ) {
       		inpar->threshold = atof(opt->getValue( "threshold") );
       	 	cout << setw(28) << " Threshold set to:  " << setw(7) << inpar->threshold << " [W/cm2]" << endl ;
       	}


       	
       	if( opt->getValue( "Ntw" ) != NULL ) {
       		inpar->Ntw = atoi(opt->getValue( "Ntw" ));
       	 	cout << setw(28) << " Ntw set to:  " << setw(7) << inpar->Ntw <<  endl ;
       	}
       	
       	if( opt->getValue( "domega" ) != NULL ) {
       		inpar->abs_omega_error = atof(opt->getValue( "domega" ));
       	 	cout << setw(28) << " domega set to:  " << setw(7) << inpar->abs_omega_error << " [omega0]" << endl ;
       	}
       	
       	  	if( opt->getValue( "taumin" ) != NULL ) {
       		inpar->taumin = atof(opt->getValue( "taumin" ));
       	 	cout << setw(28) << " taumin set to:  " << setw(7)<< inpar->taumin << " [opt.cycles]" << endl ;
       	 	inpar->taumin = 2 * PI * inpar->taumin;
       	}
       	
       	if( opt->getValue( "taumax" ) != NULL ) {
       		inpar->taumax = atof(opt->getValue( "taumax" ));
       	 	cout << setw(28) << " taumax set to:  " << setw(7)<<  inpar->taumax << " [opt.cycles]" << endl ;
       	 	inpar->taumax = 2 * PI * inpar->taumax;
       	}
       	
       	if( opt->getValue( "dtau" ) != NULL ) {
       		inpar->dtau = atof(opt->getValue( "dtau" ));
       	 	cout << setw(28) << " dtau set to:  " << setw(7)<< inpar->dtau << " [opt.cycles]" << endl ;
       	 	inpar->dtau = 2 * PI * inpar->dtau;
       	}
       	
		
       	       	
	/* 7. GET THE ACTUAL ARGUMENTS AFTER THE OPTIONS */
	for( int i = 0 ; i < opt->getArgc() ; i++ ){
		cout << "Further Arguments = " <<  opt->getArgv( i ) << endl ;
	}

        /* 8. DONE */
        delete opt;
//////////////////////////////////////////////// bsp
	return 0;
}
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


