// Author: Torsten.Leitner@email.de
// (c) 2006/2007

#include "tmptmp.h"

// tmptmp.vecpot.cpp

using namespace std;

// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

bool TestLaserBigEnoughForDipoleMoment(class tArrayComplex2D_rt *Laser , int ir, double threshold)
{
// threshold in [W/cm2]
	double PeakIntensity = 0;
	
	for ( int jt=0 ; jt<(Laser->Ntw) ; jt++ )	{  //Suche Maximum in Laser(:,ir)   [ueber alle jt]
		if ( PeakIntensity < (Laser->intensity(ir,jt)) )  PeakIntensity = (Laser->intensity(ir,jt));
	}
cout << "Peak Intensity: " << PeakIntensity << " [W/cm2]" << endl;
	if (PeakIntensity < threshold ) return false;
	return true;
}
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
inline complex<double> DipoleMatrixElement_hydrogen_like_1D(const double p, const double Ip)  // p [au], Ip[au] ;;; returns [au]
{																		
/** The dipolematrixelement for a hydrogenlike atom:\\
 *  Here only the x-Komponent is calculated\\
 *  In-Args: 	double p -Momentum ( e.g. pstat-A(t) ) [au]
 * 		double Ip - IonisationPotential [au]
 *  Out-Args:	complex diplol - dipolmatrixelement: Groundstate 0 -> Kontinuum p 
 *  S-Wavefunction: \f$\psi(\textbf{x})=\left(\frac{\2*Ip^{3/4}}{\pi^{1/2}}\right) e^{-\sqrt{\2*Ip}|\textbf{x}|}} \f$\\
 *  Dipolemoment: \textbf{d}(\textbf{p})=i\left(\frac{2^{7/2}\2*Ip^{5/4}}{\pi}\right)\frac{\textbf{p}}{(\textbf{p}^2 + \2*Ip^3)^3}\f$\\
 */
return	complex<double> ( 0 , ( pow(SQRT2,7) * pow(sqrt(sqrt(2*Ip)),5) / PI) * ( p / pow( (p*p + 2*Ip) , 3 ) )       );
}
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc



// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
int CalcVecPotAndIntegralsfullau_higherorderintegration(class tArrayComplex2D_rt *Et, Complex *A, Complex *H, Complex *Y , const int ir)
						   // H:=Int[A] --- Y:=Int[A^2]

//// momentan wird hier noch nix higher order integriert ... nur in Cacl...Xt

///   In  :	E(t) [au] ---  Out : {A(t) , H(t) , Y(t)} [au]
{
	int 	jt;
	int 	Ntw = Et->Ntw;
	double 	 dt = Et->dt;

//cout << " dt: " << dt <<endl;

	jt = 0;
	A[jt] = Complex(0,0);
	H[jt] = Complex(0,0); 
	Y[jt] = Complex(0,0);  
		 
	for (jt=1;jt<Ntw;jt++) {
		
		// Trapezoidal Rule ---   + Order[ Et'' * dt^3 ]
		A[jt] = A[jt-1] + dt * 0.5 * ( Et->at(ir,jt-1) + Et->at(ir,jt) ) ;
		H[jt] = H[jt-1] + dt * 0.5 * (  A[jt-1] +  A[jt] ); 
		Y[jt] = Y[jt-1] + dt * 0.5 * (  A[jt-1] * A[jt-1]  +  A[jt] * A[jt] ); 
		// Da VecPot und dt in au sind, sind auch H und Y in au !!!
	}

	return 0;
}
           

// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
inline Complex CalcDipMomXtfullau_integrand(const int jt, const int jy , class tArrayComplex2D_rt *Et, Complex *A, Complex *IntA, Complex *IntA2, 
					 	const int ir, const double Ip_AU, const double dt) 
{
	Complex ImagUnit(0,1);
	Complex DipIntPrefactor;
	Complex integrand;

	double Pstat,Sstat;
	double tau = dt*(jt-jy);
	   if( jt == jy ) // tau = dt*(jt-jy) = 0
		{  Pstat=0; Sstat=0;  }
	   else 
		{	
		Pstat = ( IntA[jt].real() - IntA[jy].real() ) / tau;  /// \Int_y^t {A(x)dx} / (t-y)
		// Pstat in AU, if A and tau in AU

	 	Sstat = ( Ip_AU + 0.5*Pstat*Pstat )*tau +  0.5*( IntA2[jt].real() - IntA2[jy].real() ) ;
		// Sstat in AU, if Pstat, A and tau in AU
		}
	
	   DipIntPrefactor = Complex(REGULARIZATION_CONST_TAUINTEGRAL,0.5*tau); //epsilon+i*tau/2
	   DipIntPrefactor =           pow((PI/DipIntPrefactor),1.5); // ( pi / epsilon+i*tau/2 )^(3/2)
	   
	   integrand = ImagUnit * DipIntPrefactor 
		      * 	   Et->real(ir,jy)   //jy : jt-jtau -> (t-tau) 
	              *         DipoleMatrixElement_hydrogen_like_1D(  Pstat - A[jt].real() ,  Ip_AU )
		      *  conj(  DipoleMatrixElement_hydrogen_like_1D(  Pstat - A[jy].real() ,  Ip_AU ) )
		      * 		  exp    ( -ImagUnit * Complex(Sstat,0) );	

return integrand;
}
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc           
int CalcDipoleMomentXtfullau_higherorderintegration( Complex *xt , class tArrayComplex2D_rt *Et, Complex *A, Complex *IntA, Complex *IntA2, 
					 	const int ir, const int Ntw, const double Ip_eV, const int Npoints2integrateXt) 
{
// Input : E , A,IntA,Int_A2 in au  
// Output: SI Units
	
// evtl: Conversions ...
// E[a.u.] = (1/5.14220632e11) * E[V m-1]

	double Ip_AU = Ip_eV * energy_eVtoAU;  // eV -> a.u. (Hartree) 
	double dt  = Et->dt;

//cout << " Ip AU: " << Ip_AU << " Ip eV: " << Ip_eV << "    dt: " << dt <<endl;

	Complex integral;

	int jt,jy,jymin;
	
	cout << "NL-Src : Points2Integrate = " << Npoints2integrateXt << endl;

	CalcVecPotAndIntegralsfullau_higherorderintegration( Et , A, IntA , IntA2 , ir ) ; 
		// A,IntA,IntA2 are Complex Vectors [Ntw , only valid for one ir ...

// ausm CP1 script:
//
// Wiederhohlte Trapezregel:
//
// TN = h ( 0.5*f0 + f1 + f2 + . . . + fN −1 + 0.5*fN )
//
//  I = TN + O( N^−2 ).                    
//
//
// Wiederhohlte Simpsonregel:
//
//    S2N = h (1/3 f0 + 4/3f1 + 2/3f2 + 4/3f3 + . . . + 2/3 f2N−2 + 4/3 f2N−1 + 1/3f2N )
//
// mit h = (b − a)/(2N ). Das ist gerade die wiederholte Simpson–Regel,
// die je Teilintervall eine Genauigkeit O(h5 ) aufweist, also nach Summation
// immer noch eine Konvergenz der Ordnung N −4 erreicht:
//                              I = S2N + O(N^−4 ).                    
// Wiederhohlte Simpsonregel kann auch durch wdh.Traperegel dargestellt werden:
// 
//	S2N = (4/3)*T2N  - (1/3)*TN
// 


	for (jt=0;jt<Ntw;jt++) {  /// for all t ... Calc x(t)
	
	  if (jt%1000==0) {cout << "NL-Src : x(t)::ir = " << ir << " -  x(t)::jt = " << jt << endl;}	 
	   
	  integral   = 0;
	  
	  jymin  =  jt  -   Npoints2integrateXt ;  // Integrate only over several opt.cycles, thus not over all points  
	  		if(jymin<0){jymin=0;} 

// Simpson Rule ...
//	S2N = h (1/3 f0 + 4/3f1 + 2/3f2 + 4/3f3 + . . . + 2/3 f2N−2 + 4/3 f2N−1 + 1/3f2N )

	  jy=jymin;

 	  if ( (jymin-jt)%2 != 0 ) { // if number of points is odd,
		// then calculate first step with trapezoidal rule ...
	     integral +=  3*0.5*( CalcDipMomXtfullau_integrand(jt, jy  , Et, A, IntA, IntA2, ir, Ip_AU, dt) 
			   +      CalcDipMomXtfullau_integrand(jt, jy+1, Et, A, IntA, IntA2, ir, Ip_AU, dt)  );
	     /// *3, because the integral is divided by three in the end, due to the simpson rule
	     jy++;
	  }

	// first 1/3 part
	  integral +=  CalcDipMomXtfullau_integrand(jt, jy, Et, A, IntA, IntA2, ir, Ip_AU, dt) ;
	  jy++;
	  while (jy<(jt-1)) { /// x(t) = \int_{-inf}^{t} f(y)dy --- y=-inf -> jy=0; y=t -> jy=jt  

	// 4/3 parts
		integral += Complex(4,0)*CalcDipMomXtfullau_integrand(jt, jy, Et, A, IntA, IntA2, ir, Ip_AU, dt) ;
		jy++;	
	// 2/3 parts
		integral += Complex(2,0)*CalcDipMomXtfullau_integrand(jt, jy, Et, A, IntA, IntA2, ir, Ip_AU, dt);
		jy++;

	  } // end while jy<jt

	// last 4/3 part
	  integral += Complex(4,0)*CalcDipMomXtfullau_integrand(jt, jy,   Et, A, IntA, IntA2, ir, Ip_AU, dt) ;
	  jy++;
	// last 1/3 part
	  integral +=  CalcDipMomXtfullau_integrand(jt, jy, Et, A, IntA, IntA2, ir, Ip_AU, dt) ;

	// *h/3	
	  xt[jt] =  (dt/3)*integral;

	} // end for jt

	cout << "NL-Src : x(t)::ir = " << ir << " -  x(t)::jt = " << jt << endl;
	return 0;
}	
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
int CalcVecPotAndIntegralsfullau(class tArrayComplex2D_rt *Et, Complex *A, Complex *H, Complex *Y , const int ir)
						   // H:=Int[A] --- Y:=Int[A^2]
///   In  :	E(t) [au] ---  Out : {A(t) , H(t) , Y(t)} [au]
{
	int 	jt;
	int 	Ntw = Et->Ntw;
	double 	 dt = Et->dt;

//cout << " dt: " << dt <<endl;

	jt = 0;
	A[jt] = Complex(0,0);
	H[jt] = Complex(0,0); 
	Y[jt] = Complex(0,0);  
		 
	for (jt=1;jt<Ntw;jt++) {
		
		// Trapezoidal Rule ---   + Order[ Et'' * dt^3 ]
		A[jt] = A[jt-1] + dt * 0.5 * ( Et->at(ir,jt-1) + Et->at(ir,jt) ) ;
		H[jt] = H[jt-1] + dt * 0.5 * (  A[jt-1] +  A[jt] ); 
		Y[jt] = Y[jt-1] + dt * 0.5 * (  A[jt-1] * A[jt-1] +  A[jt] * A[jt] ); 
		// Da VecPot und dt in au sind, sind auch H und Y in au !!!
	}

	return 0;
}
           
// new variant with external integrand-function ...
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc           
int CalcDipoleMomentXtfullau( Complex *xt , class tArrayComplex2D_rt *Et, Complex *A, Complex *IntA, Complex *IntA2, 
					 	const int ir, const int Ntw, const double Ip_eV, const int Npoints2integrateXt) 
{
// Input : E , A,IntA,Int_A2 in au  
// Output: SI Units
	
// evtl: Conversions ...
// E[a.u.] = (1/5.14220632e11) * E[V m-1]

	double Ip_AU = Ip_eV * energy_eVtoAU;  // eV -> a.u. (Hartree) 
	double dt  = Et->dt;

//cout << " Ip AU: " << Ip_AU << " Ip eV: " << Ip_eV << "    dt: " << dt <<endl;

	Complex integral;

	int jt,jy,jymin;
	
	cout << "NL-Src : Points2Integrate = " << Npoints2integrateXt << endl;

	CalcVecPotAndIntegralsfullau_higherorderintegration( Et , A, IntA , IntA2 , ir ) ; 
		// A,IntA,IntA2 are Complex Vectors [Ntw , only valid for one ir ...

// ausm CP1 script:
//
// Wiederhohlte Trapezregel:
//
// TN = h ( 0.5*f0 + f1 + f2 + . . . + fN −1 + 0.5*fN )
//
//  I = TN + O( N^−2 ).                    
//
//
// Wiederhohlte Simpsonregel:
//
//    S2N = h (1/3 f0 + 4/3f1 + 2/3f2 + 4/3f3 + . . . + 2/3 f2N−2 + 4/3 f2N−1 + 1/3f2N )
//
// mit h = (b − a)/(2N ). Das ist gerade die wiederholte Simpson–Regel,
// die je Teilintervall eine Genauigkeit O(h5 ) aufweist, also nach Summation
// immer noch eine Konvergenz der Ordnung N −4 erreicht:
//                              I = S2N + O(N^−4 ).                    
// Wiederhohlte Simpsonregel kann auch durch wdh.Traperegel dargestellt werden:
// 
//	S2N = (4/3)*T2N  - (1/3)*TN
// 


	for (jt=0;jt<Ntw;jt++) {  /// for all t ... Calc x(t)
	
	  if (jt%1000==0) {cout << "NL-Src : x(t)::ir = " << ir << " -  x(t)::jt = " << jt << endl;}	 
	   
	  integral   = 0;
	  
	  jymin  =  jt  -   Npoints2integrateXt ;  // Integrate only over several opt.cycles, thus not over all points  
	  		if(jymin<0){jymin=0;} 
// Wdh. Trapezregel ...
	  jy=jymin;
	  integral  += Complex(0.5,0) * CalcDipMomXtfullau_integrand(jt, jy, Et, A, IntA, IntA2, ir, Ip_AU, dt) ;
    	  jy+=1;
	  while (jy<jt) { /// x(t) = \int_{-inf}^{t} f(y)dy --- y=-inf -> jy=0; y=t -> jy=jt  

	        integral  += CalcDipMomXtfullau_integrand(jt, jy, Et, A, IntA, IntA2, ir, Ip_AU, dt) ;
	    	jy+=1;

	  } // end while jy<jt
	  integral  += Complex(0.5,0) * CalcDipMomXtfullau_integrand(jt, jy, Et, A, IntA, IntA2, ir, Ip_AU, dt) ;	

	  xt[jt] =  dt*integral;

	} // end for jt

	cout << "NL-Src : x(t)::ir = " << ir << " -  x(t)::jt = " << jt << endl;
	return 0;
}	
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
//*/


// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
/*
// old variant with integrand-function included...
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc           
int CalcDipoleMomentXtfullau( Complex *xt , class tArrayComplex2D_rt *Et, Complex *A, Complex *IntA, Complex *IntA2, 
					 	const int ir, const int Ntw, const double Ip_eV, const int Npoints2integrateXt) 
{
// Input : E , A,IntA,Int_A2 in au  
// Output: SI Units
	
// evtl: Conversions ...
// E[a.u.] = (1/5.14220632e11) * E[V m-1]

	double Ip_AU = Ip_eV * energy_eVtoAU;  // eV -> a.u. (Hartree) 
	double dt  = Et->dt;

//cout << " Ip AU: " << Ip_AU << " Ip eV: " << Ip_eV << "    dt: " << dt <<endl;

	Complex DipIntPrefactor;
	Complex integral,integrand1 ,integrand2;

	Complex ImagUnit(0,1);

	double tau,Pstat,Sstat;

	int jt,jy,jymin;
	
	cout << "NL-Src : Points2Integrate = " << Npoints2integrateXt << endl;

	CalcVecPotAndIntegralsfullau( Et , A, IntA , IntA2 , ir ) ; // A,IntA,IntA2 are Complex Vectors [Ntw , only valid for one ir ...

	for (jt=0;jt<Ntw;jt++) {  /// for all t ... Calc x(t)
	
	  if (jt%1000==0) {cout << "NL-Src : x(t)::ir = " << ir << " -  x(t)::jt = " << jt << endl;}	 
	   
	  integral   = 0;
	  integrand1 = 0;
	  integrand2 = 0;
	  
	  jymin  =  jt  -   Npoints2integrateXt ;  // Integrate only over several opt.cycles, thus not over all points  
	  		if(jymin<0){jymin=0;} 
	  
	  for (jy=jymin;jy<=jt;jy++) { /// x(t) = \int_{-inf}^{t} f(y)dy --- y=-inf -> jy=0; y=t -> jy=jt  
	
	   tau = dt*(jt-jy);
	   if( jt == jy ) // tau = dt*(jt-jy) = 0
		{  Pstat=0; Sstat=0;  }
	   else 
		{	
		Pstat = ( IntA[jt].real() - IntA[jy].real() ) / tau;  /// \Int_y^t {A(x)dx} / (t-y)
		// Pstat in AU, if A and tau in AU

	 	Sstat = ( Ip_AU + 0.5*Pstat*Pstat )*tau +  0.5*( IntA2[jt].real() - IntA2[jy].real() ) ;
		// Sstat in AU, if Pstat, A and tau in AU
		}
	
	   DipIntPrefactor = Complex(REGULARIZATION_CONST_TAUINTEGRAL,0.5*tau); //epsilon+i*tau/2
	   DipIntPrefactor =           pow((PI/DipIntPrefactor),1.5); // ( pi / epsilon+i*tau/2 )^(3/2)
	   
	   integrand1 = integrand2;
	   integrand2 = ImagUnit * DipIntPrefactor 
		      * 	   Et->real(ir,jy)   //jy : jt-jtau -> (t-tau) 
	              *         DipoleMatrixElement_hydrogen_like_1D(  Pstat - A[jt].real() ,  Ip_AU )
		      *  conj(  DipoleMatrixElement_hydrogen_like_1D(  Pstat - A[jy].real() ,  Ip_AU ) )
		      * 		  exp    ( -ImagUnit * Sstat );	

//	   integral += dt * integrand;
	   integral +=  ( integrand1 + integrand2 );
	   /// Evtl verbessertes Integral ... Simpson oder so... jeweils n-integranden zwischenspeichern ...
	   ///  jy muss nicht bei 0 beginnen, jt eigentlich auch nicht... E(jt<<)~0	
	  } // end for jy
	
// Convert to SI units
//	  xt[jt] = electric_dipole_moment_AUtoSI * dt* integral;
	  xt[jt] =  0.5 *dt*integral;

	} // end for jt

	cout << "NL-Src : x(t)::ir = " << ir << " -  x(t)::jt = " << jt << endl;
	return 0;
}	
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

//*/


// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc



// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
// From here.. with full fields A intA and so on ...
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
// Dummy function, which calls the selected integraton method ...
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
int CalcVecPotAndIntegrals(class tArrayComplex2D_rt *Et, class tArrayComplex2D_rt *A,
						   class tArrayComplex2D_rt *H,  class tArrayComplex2D_rt *Y )
{						   
	//CalcVecPotAndIntegrals_fft(Et, A, H, Y);
	CalcVecPotAndIntegrals_int(Et, A, H, Y);	
	return 0;
}					  
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc           
int CalcVecPotAndIntegrals_int(class tArrayComplex2D_rt *Et, class tArrayComplex2D_rt *A,
						   class tArrayComplex2D_rt *H,  class tArrayComplex2D_rt *Y )
						   // H:=Int[A] --- Y:=Int[A^2]
///   In  :	E(t) [SI] ---  Out : {A(t) , H(t) , Y(t)} [SI]
{
	int jt,ir;
	
	int 	Nr  = Et->Nr;
	int 	Ntw = Et->Ntw;
	double 	 dt = Et->dt ;//* time_SItoAU; // dt in au !!!
	
	for (ir=0;ir<Nr;ir++) {
		
		jt = 0;
		A->data[ir][jt] = Complex(0,0);
		H->data[ir][jt] = Complex(0,0); 
		Y->data[ir][jt] = Complex(0,0);  
		 
		for (jt=1;jt<Ntw;jt++) {
			
			// Trapezoidal Rule ---   + Order[ Et'' * dt^3 ]
			A->data[ir][jt] = A->at(ir,jt-1) + dt * 0.5 * ( Et->at(ir,jt-1) + Et->at(ir,jt) ) ;//* VperM_SItoAU ; // Vecpot in au !!!
			H->data[ir][jt] = H->at(ir,jt-1) + dt * 0.5 * (  A->at(ir,jt-1) +  A->at(ir,jt) ); 
			Y->data[ir][jt] = Y->at(ir,jt-1) + dt * 0.5 * (  A->at(ir,jt-1)* A->at(ir,jt-1) +  A->at(ir,jt)* A->at(ir,jt)); 
		// Da VecPot und dt in au sind auch H und Y in au !!!
		}
	}
	
	
	return 0;
}
           

// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc



// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc           
int CalcVecPotAndIntegrals_fft(class tArrayComplex2D_rt *Et, class tArrayComplex2D_rt *A,
						   class tArrayComplex2D_rt *H,  class tArrayComplex2D_rt *Y )
						   // H:=Int[A] --- Y:=Int[A^2]
///   In  :	E(t)  ---  Out : A(t) , H(t) , Y(t)
/// everything in SI-Units
{
	
	
	int Nr  = Et->Nr;
	int Ntw = Et->Ntw;
	
	for (int ir=0;ir<Nr;ir++) {
		for (int jt=1;jt<Ntw-1;jt++) {
			H->data[ir][jt] = Et->data[ir][jt] ;// H'(t)= E(t) 
		}
	}
	H->fft(); // H = E(w) ,,, E = E(t)
	/// A(t) , H(t) ...
	for (int ir=0;ir<Nr;ir++) {
		for (int jt=1;jt<Ntw-1;jt++) {
			A->data[ir][jt] =  H->data[ir][jt] / Complex( 0 , H->omega[jt] ) ;		// A(w)=  E(w)/iw
			H->data[ir][jt] =  H->data[ir][jt] / ( - H->omega[jt] * H->omega[jt] ) ;// H(w)= -E(w)/w^2
		}
		A->data[ir][0]     =  Complex(0,0) ; 	  // w=0
		A->data[ir][Ntw-1] =  Complex(0,0) ; // w=0 
		A->data[ir][Ntw-2] =  Complex(0,0) ; // This shouldn't be ... check omega-vector in class "myarray"
		H->data[ir][0]     =  Complex(0,0) ; 	  // w=0
		H->data[ir][Ntw-1] =  Complex(0,0) ; // w=0 
		H->data[ir][Ntw-2] =  Complex(0,0) ; // This shouldn't be ... check omega-vector in class "myarray"
	} 
	A->ifft();  // A(t) = ifft[A(w)] = ifft[ E(w)/iw]
	H->ifft();  // H(t) = ifft[H(w)] = ifft[-E(w)/w^2]
	// Y(t) ...
	for (int ir=0;ir<Nr;ir++) {
		for (int jt=1;jt<Ntw-1;jt++) {
			Y->data[ir][jt] =  A->data[ir][jt] * A->data[ir][jt];	// Y'(t) = A^2(t) 
		}
	}			
	Y->fft();														// Y'(w) = fft[A^2(t)]
	for (int ir=0;ir<Nr;ir++) {
		for (int jt=1;jt<Ntw-1;jt++) {								// Y (w) =  Y'(w)/iw
			Y->data[ir][jt] =  Y->data[ir][jt] / Complex( 0 , H->omega[jt] ) ;		 
		}															
		Y->data[ir][0]     =  Complex(0,0) ; 	  // w=0
		Y->data[ir][Ntw-1] =  Complex(0,0) ; // w=0 
		Y->data[ir][Ntw-2] =  Complex(0,0) ; // This shouldn't be ... check omega-vector in class "myarray"
	}
	Y->ifft();	// Y(t) = ifft[Y(w)] = ifft[Y'(w)/iw] = ifft[ fft[A^2(t)] /iw]
	
/// Et=E(t) , A=A(t) , H=H(t) , Y=Y(t) 
	
	return 0;
}
           

// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc           
int CalcDipoleMoment( 	Complex *xt ,
						class tArrayComplex2D_rt *Et,   class tArrayComplex2D_rt *A, 
						class tArrayComplex2D_rt *IntA, class tArrayComplex2D_rt *IntA2, 
					 	const int ir, const int Ntw1, const double dt_SI, 
					 	const double omega0, const double Ip_eV) 
{
// Input : E in SI , A,IntA,Int_A2 in au ; dt_SI; Ip_eV; omega0 in SI	
// Output: SI Units
	
// evtl: Conversions ...
// E[a.u.] = (1/5.14220632e11) * E[V m-1]

	int NopticalCyclesToIntegrate = 3; // Anzahl der opt.Lasercyclen uber die x(t) ausgwertet wird
						// Anz. der opt cyc. die ein elektron unterwegs ist.

//	double Ip_SI = ECHARGE*Ip_eV;  // IpSI = ECHARGE * Ip[eV]
	double Ip_AU = Ip_eV * energy_eVtoAU;  // eV -> a.u. (Hartree) 
	
	double dt_AU = dt_SI * time_SItoAU ;

	Complex DipIntPrefactor;
	Complex integral,integrand1 ,integrand2;

	Complex ImagUnit(0,1);

	double tau,Pstat,Sstat;

	int jt,jy,jymin;
	
	int PointsPerOpticalCycle = int( 2*PI/(omega0*dt_SI) ) + 1 ; // +1 because int(x) always rounds down ...
	 
	cout << "PpOC : " << PointsPerOpticalCycle << endl;
	cout << "I integrate over 3 opt. cycles "  << endl;

	cout << "NL-Src : x(t)::ir = " << ir << endl;

	for (jt=0;jt<Ntw1;jt++) {  /// for all t ... Calc x(t)
	
	  if (jt%1000==0) {cout << "NL-Src : x(t)::ir = " << ir << " -  x(t)::jt = " << jt << endl;}	 
	   
	  integral   = 0;
	  integrand1 = 0;
	  integrand2 = 0;
	  
	  jymin  =  jt  -   NopticalCyclesToIntegrate * PointsPerOpticalCycle ;  // Integrate only over several opt.cycles  
	  		if(jymin<0){jymin=0;} 
	  
	  for (jy=jymin;jy<jt;jy++) { /// x(t) = \int_{-inf}^{t} f(y)dy --- y=-inf -> jy=0; y=t -> jy=jt  
	
	   tau = dt_AU*(jt-jy);
	   if( jt == jy ) // tau = dt*(jt-jy) = 0
		{  Pstat=0; Sstat=0;  }
	   else 
		{	
		
		Pstat = vecpot_SItoAU*( IntA->real(ir,jt)-IntA->real(ir,jy) ) / tau;  /// \Int_y^t {A(x)dx} / (t-y)
		// Pstat in AU, if A and tau in AU
		
	 	Sstat = ( Ip_AU + 0.5*Pstat*Pstat )*tau +  0.5*vecpot_SItoAU*vecpot_SItoAU*(IntA2->real(ir,jt)-IntA2->real(ir,jy)) ;
		// Sstat in AU, if Pstat, A and tau in AU
//		Sstat = Sstat * action_SItoAU;   // Sstat SI to AU
 
		 
		}
	
	   DipIntPrefactor = Complex(REGULARIZATION_CONST_TAUINTEGRAL,0.5*tau); //epsilon+i*tau/2
	   DipIntPrefactor =           pow((PI/DipIntPrefactor),1.5); // ( pi / epsilon+i*tau/2 )^(3/2)
	   
	   integrand1 = integrand2;
	   integrand2 = ImagUnit * DipIntPrefactor 
		      * 	     VperM_SItoAU * Et->real(ir,jy)   //jy : jt-jtau -> (t-tau) 
	          *         DipoleMatrixElement_hydrogen_like_1D(  Pstat - vecpot_SItoAU*A->real(ir,jt) ,  Ip_AU )
		      *  conj(  DipoleMatrixElement_hydrogen_like_1D(  Pstat - vecpot_SItoAU*A->real(ir,jy) ,  Ip_AU ) )
		      * 		  exp    ( -ImagUnit * Sstat );	

//	   integral += dt * integrand;
	   integral += 0.5 * dt_AU * ( integrand1 + integrand2 );
	   /// Evtl verbessertes Integral ... Simpson oder so... jeweils n-integranden zwischenspeichern ...
	   ///  jy muss nicht bei 0 beginnen, jt eigentlich auch nicht... E(jt<<)~0	
	  } // end for jy
	
// Convert to SI units
	  xt[jt] = electric_dipole_moment_AUtoSI * integral;

	} // end for jt

	cout << "NL-Src : x(t)::ir = " << ir << " -  x(t)::jt = " << jt << endl;
	return 0;
}	
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc           
int CalcNonLinSrc(class tArrayComplex2D_rt *Et, class tArrayComplex2D_rt *SrcH, const double omega0, const double Ip) 
/// In : Laser(t) --- Out : NonLinSrc(w)
{

	int      Nr1 = Et->Nr,     Ntw1 = Et->Ntw;
	double rmax1 = Et->rmax , wmax1 = Et->wmax,  dt = Et->dt;

	int      NrH = SrcH->Nr,     NtwH = SrcH->Ntw;
	
	double rmaxH = SrcH->rmax , wmaxH = SrcH->wmax ;//, wminH=SrcH->wmin;

//2**19=524288//2**18=262144//2**17=131072//2**16=65536//2**15=32768
//2**14= 16384//2**13=  8192//2**12=  4096//2**11= 2048//2**10= 1024
	
	//// evtl struct mit Harmonic Params...w0,wmax,q,etc oder alles ueber inpar ... ???

	// *****  Felder A(t), H(t)=Int[A(t], Y(t)=Int[A^2(t)]  ********
	class tArrayComplex2D_rt  A     (Nr1,rmax1,Ntw1,wmax1);
	class tArrayComplex2D_rt  IntA  (Nr1,rmax1,Ntw1,wmax1);
	class tArrayComplex2D_rt  IntA2 (Nr1,rmax1,Ntw1,wmax1);

	int   NPw = NtwH; // erweiterter w-Raum, um hohe Frequenzen abbilden zu koennen ...
	wmaxH = 1; // Anpassen... Qmax*w0 oder so... sonst w-Vektor in P falsch ...
	class tArrayComplex2D_rt  P     (NrH,rmaxH,NPw,wmaxH);
	// *************************************************************

	//Complex DipIntPrefactor;	Complex integral,integrand1 ,integrand2;	Complex ImagUnit(0,1);

	Complex *xt;  xt = new Complex[Ntw1];


	int ir,jt;
  // ******** END DECLARATIONS ***********************************


	CalcVecPotAndIntegrals(Et, &A, &IntA, &IntA2);  	   // H:=Int[A] --- Y:=Int[A^2]

	/////   In  :	Et(t)  ---  Out : A(t) , IntA(t) , IntA2(t)

//	double alpha=2*Ip; //from P.Antoine et.al. -> PhRevA,Vol.53#3,March1996

	for (ir=0;ir<Nr1;ir++) {

		CalcDipoleMoment(xt, Et, &A, &IntA, &IntA2, ir, Ntw1, dt, omega0 , Ip); 	
///  Wir haben jetzt xt(r) ...

// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
///	Kopiere (n0-ne(r,z,t))*xt in mitte von P(w) ...  vorerst ne=0; n0=1
	for (jt=0;jt<int((NPw-Ntw1)/2) ;jt++){	P.data[ir][jt] = 0 ; }
	for (jt=0;jt<Ntw1;jt++){	
						P.data[ir][jt+int((NPw-Ntw1)/2)] = Complex(2* xt[jt].real() , 0 );   
					//	P.data[ir][jt+int((NPw-Ntw1)/2)] = xt[jt];   
		    }
	for (jt=int((NPw-Ntw1)/2)+Ntw1;jt<NPw ;jt++){	P.data[ir][jt] = 0 ; }
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


}  // end for ir , Nr1

P.ifft();  // P=P(w);

for (jt=0;jt<NtwH;jt++) {
//	SrcH->omega[jt] = jt;
	for (ir=0;ir<Nr1;ir++) {
		SrcH->data[ir][jt] = P.data[ir][jt];
//		SrcH->data[ir][jt] = -MUE0*P.omega[jt+qmin]*P.omega[jt+qmin]*P.data[ir][jt+qmin];
	}
}  // end for ir , Nr1

	return 0;
}
// ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc





